@@ -269,8 +269,8 @@ pub fn resolve_path_cached(
 
 pub fn initialize_hier(connection: &UpEndConnection) -> Result<()> {
     connection.insert_entry(Entry::try_from(&*HIER_INVARIANT)?)?;
-    upend_insert_addr!(connection, HIER_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR);
-    upend_insert_val!(connection, HIER_ADDR, TYPE_HAS_ATTR, HIER_HAS_ATTR);
+    upend_insert_addr!(connection, HIER_ADDR, IS_OF_TYPE_ATTR, TYPE_ADDR)?;
+    upend_insert_val!(connection, HIER_ADDR, TYPE_HAS_ATTR, HIER_HAS_ATTR)?;
     Ok(())
 }
 
