@@ -20,12 +20,10 @@ use crate::database::UpEndDatabase;
 mod addressing;
 mod database;
 mod filesystem;
+mod previews;
 mod routes;
 mod util;
 
-#[cfg(feature = "previews")]
-mod previews;
-
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 fn main() -> Result<()> {
@@ -99,6 +97,14 @@ fn main() -> Result<()> {
 
     let upend = Arc::new(open_result.db);
 
+    let ui_path = env::current_exe().unwrap().parent().unwrap().join("webui");
+    if !ui_path.exists() {
+        warn!("No Web UI directory present ({:?}), disabling...", ui_path);
+    }
+    let desktop_enabled = !matches.is_present("NO_DESKTOP");
+    let ui_enabled = ui_path.exists() && !matches.is_present("NO_UI");
+    let browser_enabled = desktop_enabled && !matches.is_present("NO_BROWSER");
+
     #[cfg(feature = "previews")]
     let preview_store = Some(Arc::new(crate::previews::PreviewStore::new(
         upend.db_path.join("previews"),
@@ -131,14 +137,10 @@ fn main() -> Result<()> {
         ),
         job_container: job_container.clone(),
         preview_store,
+        desktop_enabled,
     };
 
     // Start HTTP server
-    let ui_path = env::current_exe().unwrap().parent().unwrap().join("webui");
-    if !ui_path.exists() {
-        warn!("No Web UI directory present ({:?}), disabling...", ui_path);
-    }
-    let ui_enabled = ui_path.exists() && !matches.is_present("NO_UI");
 
     let mut cnt = 0;
     let server = loop {
@@ -147,6 +149,7 @@ fn main() -> Result<()> {
 
         let server = HttpServer::new(move || {
             let app = App::new()
+                .app_data(actix_web::web::PayloadConfig::new(4_294_967_296))
                 .data(state.clone())
                 .wrap(middleware::Logger::default().exclude("/api/jobs"))
                 .service(routes::get_raw)
@@ -155,6 +158,7 @@ fn main() -> Result<()> {
                 .service(routes::get_object)
                 .service(routes::put_object)
                 .service(routes::delete_object)
+                .service(routes::get_all_attributes)
                 .service(routes::api_refresh)
                 .service(routes::list_hier)
                 .service(routes::list_hier_roots)
@@ -190,12 +194,12 @@ fn main() -> Result<()> {
 
     if !matches.is_present("NO_INITIAL_UPDATE") {
         info!("Running initial update...");
-        actix::spawn(filesystem::rescan_vault(upend, job_container));
+        actix::spawn(filesystem::rescan_vault(upend, job_container, true));
     }
 
     #[cfg(feature = "desktop")]
     {
-        if !matches.is_present("NO_BROWSER") && ui_enabled {
+        if browser_enabled && ui_enabled {
             let ui_result = webbrowser::open(&format!("http://localhost:{}", bind.port()));
             if ui_result.is_err() {
                 warn!("Could not open UI in browser!");
