@@ -1,9 +1,9 @@
 use crate::addressing::{Address, Addressable};
 use crate::database::entry::{Entry, InEntry};
 use crate::database::hierarchies::{list_roots, resolve_path, UHierPath};
-use crate::database::inner::models;
 use crate::database::lang::Query;
 use crate::database::UpEndDatabase;
+use crate::filesystem::add_file;
 use crate::previews::PreviewStore;
 use crate::util::hash::{decode, encode, Hashable};
 use crate::util::jobs::JobContainer;
@@ -13,7 +13,6 @@ use actix_web::error::{ErrorBadRequest, ErrorInternalServerError, ErrorNotFound}
 use actix_web::http;
 use actix_web::{delete, error, get, post, put, web, Either, Error, HttpResponse};
 use anyhow::{anyhow, Result};
-use chrono::{NaiveDateTime, Utc};
 use futures_util::TryStreamExt;
 use log::{debug, info, trace};
 use serde::Deserialize;
@@ -23,7 +22,6 @@ use std::fs;
 use std::io::Write;
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
-use std::time::UNIX_EPOCH;
 use std::{collections::HashMap, io};
 use tempfile::NamedTempFile;
 
@@ -216,53 +214,37 @@ pub async fn put_object(
                 }
                 let path = PathBuf::from(file.path());
                 let hash = web::block(move || path.hash()).await?;
-
                 let address = Address::Hash(hash.clone());
 
-                let addr_str = encode(address.encode().map_err(ErrorInternalServerError)?);
-                let final_name = if let Some(filename) = filename {
-                    format!("{addr_str}_{filename}")
-                } else {
-                    addr_str
-                };
-
-                let final_path = state.upend.vault_path.join(&final_name);
-
-                let (_, tmp_path) = file.keep().map_err(ErrorInternalServerError)?;
-                let final_path = web::block::<_, _, io::Error>(move || {
-                    fs::copy(&tmp_path, &final_path)?;
-                    fs::remove_file(tmp_path)?;
-                    Ok(final_path)
-                })
-                .await?;
-
-                let metadata = fs::metadata(&final_path)?;
-                let size = metadata.len() as i64;
-                let mtime = metadata
-                    .modified()
-                    .map(|t| {
-                        NaiveDateTime::from_timestamp(
-                            t.duration_since(UNIX_EPOCH).unwrap().as_secs() as i64,
-                            0,
-                        )
-                    })
-                    .ok();
-
                 let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
 
-                connection
-                    .insert_file(models::NewFile {
-                        path: final_name,
-                        hash: hash.0,
-                        added: NaiveDateTime::from_timestamp(Utc::now().timestamp(), 0),
-                        size,
-                        mtime,
-                    })
+                let existing_files = connection
+                    .retrieve_file(hash.clone())
                     .map_err(ErrorInternalServerError)?;
 
+                if existing_files.is_empty() {
+                    let addr_str = encode(address.encode().map_err(ErrorInternalServerError)?);
+                    let final_name = if let Some(filename) = filename {
+                        format!("{addr_str}_{filename}")
+                    } else {
+                        addr_str
+                    };
+
+                    let final_path = state.upend.vault_path.join(&final_name);
+
+                    let (_, tmp_path) = file.keep().map_err(ErrorInternalServerError)?;
+                    let final_path = web::block::<_, _, io::Error>(move || {
+                        fs::copy(&tmp_path, &final_path)?;
+                        fs::remove_file(tmp_path)?;
+                        Ok(final_path)
+                    })
+                    .await?;
+
+                    add_file(&connection, &final_path, hash).map_err(ErrorInternalServerError)?;
+                }
+
                 if let Some(filename) = filename {
-                    upend_insert_val!(&connection, address, "LBL", filename)
-                        .map_err(ErrorInternalServerError)?;
+                    let _ = upend_insert_val!(&connection, address, "LBL", filename);
                 }
 
                 Ok((address, None))
