@@ -4,7 +4,12 @@ export type VALUE_TYPE = "Value" | "Address" | "Invalid";
 export interface IEntry {
   entity: Address;
   attribute: string;
-  value: { t: VALUE_TYPE; c: string | number };
+  value: IValue;
+}
+
+export interface IValue {
+  t: VALUE_TYPE;
+  c: string | number;
 }
 
 export interface ListingResult {
@@ -28,8 +33,7 @@ export interface IJob {
   state: "InProgress" | "Done" | "Failed";
 }
 
-
 export interface VaultInfo {
   name: string | null;
   location: string;
-}
\ No newline at end of file
+}
